//
//  GroupFilterViewController.m
//  SDKTestApp
//
//  Created by Vishwesh Joshi on 06/11/15.
//  Copyright © 2015 Robosoft. All rights reserved.
//

#import "GroupFilterViewController.h"
#import <TeamchatSDK/TeamchatSDK.h>
#import <TeamchatSDK/Teamchat.h>
#import "ListTableViewController.h"
#import "SVProgressHUD.h"

typedef NS_ENUM(NSInteger, GroupFilterTableSection)
{
    GroupFilterTableSectionEmail,
    GroupFilterTableSectionGroup
};
@interface GroupFilterViewController () <UITableViewDataSource,UITableViewDelegate,ListControllerDelegate>
@property (weak, nonatomic) IBOutlet UISwitch *includeSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *ExcludeSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *oneToOneOnlySwitch;
@property (weak, nonatomic) IBOutlet UISwitch *groupsOnlySwitch;
@property (weak, nonatomic) IBOutlet UIButton *groupsToIncludeOrExcludeButton;
@property (weak, nonatomic) IBOutlet UITableView *SelectGroupsTableView;
@property (weak, nonatomic) IBOutlet UIButton *resetButto;
@property (weak, nonatomic) IBOutlet UITextField *emailEnteringTextField;
@property (weak, nonatomic) IBOutlet UIButton *addEmail;
@property (weak, nonatomic) IBOutlet UITextField *emailTextField;
@property (nonatomic,strong) UIViewController * groupsViewController;
@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;

@end

@implementation GroupFilterViewController
{
    NSMutableArray * allSelectedGroups;
    NSMutableArray * allSelectedEmails;
}
- (void)viewDidLoad
{
    [super viewDidLoad];
    allSelectedGroups = [NSMutableArray new];
    allSelectedEmails = [NSMutableArray new];

    // Calling Initialization for safety
    [SVProgressHUD show];
    [Teamchat initWithCompletionHandler:^(BOOL success, NSError *error, NSString *errMsg) {
        [SVProgressHUD dismiss];
    }];
    
    self.title = @"Group Filter";
    UIBarButtonItem *barButton = [[UIBarButtonItem alloc] init];
    barButton.title = @"Done";
    self.navigationController.navigationBar.topItem.backBarButtonItem = barButton;
    
    self.SelectGroupsTableView.delegate = self;
    self.SelectGroupsTableView.dataSource = self;
    
    [self resetFilters];
}

-(void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self registerToKeyboardNotifications];
}

-(void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

-(void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    if (self.emailTextField.isFirstResponder)
    {
        [self.emailTextField resignFirstResponder];
    }
    [self unregisterToKeyboardNotifications];
}


- (void)registerToKeyboardNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardDidShow:)
                                                 name:UIKeyboardWillShowNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillBeHidden:)
                                                 name:UIKeyboardWillHideNotification
                                               object:nil];
}
- (void)unregisterToKeyboardNotifications
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillHideNotification object:nil];
}

#pragma mark - Keyboard Handling

- (void)keyboardDidShow:(NSNotification *)notification
{
    NSDictionary* info = [notification userInfo];
    CGRect kbRect = [[info objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue];
    kbRect = [self.view convertRect:kbRect fromView:nil];
    
    UIEdgeInsets contentInsets = UIEdgeInsetsMake(0.0, 0.0, kbRect.size.height, 0.0);
    self.scrollView.contentInset = contentInsets;
    self.scrollView.scrollIndicatorInsets = contentInsets;
    
    CGRect aRect = self.view.frame;
    aRect.size.height -= kbRect.size.height;
    if (!CGRectContainsPoint(aRect, self.emailTextField.frame.origin) ) {
        [self.scrollView scrollRectToVisible:self.emailTextField.frame animated:YES];
    }
}

- (void)keyboardWillBeHidden:(NSNotification *)notification
{
    [self.scrollView setContentInset:UIEdgeInsetsMake(self.navigationController.navigationBar.bounds.size.height + [[UIApplication sharedApplication] statusBarFrame].size.height, 0.0, 0.0, 0.0)];
}

-(void)dealloc
{
    [self resetFilters];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (section == GroupFilterTableSectionEmail)
    {
        return allSelectedEmails.count;
    }
    if (section == GroupFilterTableSectionGroup)
    {
        return allSelectedGroups.count;
    }
    return 0;
}
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 2;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell * cell ;
    if (indexPath.section == GroupFilterTableSectionEmail)
    {
        cell = [tableView dequeueReusableCellWithIdentifier:@"GroupsFilterSelectedCellIdentifier" forIndexPath:indexPath];
        cell.textLabel.text = allSelectedEmails[indexPath.row];
        cell.detailTextLabel.text = nil;
    }
    if (indexPath.section == GroupFilterTableSectionGroup)
    {
        cell = [tableView dequeueReusableCellWithIdentifier:@"GroupsFilterSelectedCellIdentifier" forIndexPath:indexPath];
        
        TeamchatGroup *group = allSelectedGroups[indexPath.row];
        
        cell.textLabel.text = [NSString stringWithFormat:@"Group Name : %@", group.name];
        cell.detailTextLabel.text = [NSString stringWithFormat:@"Group ID : %@", group.groupID];
    }
   
    return cell;
}


- (IBAction)includeSwitchStateChanged:(UISwitch *)sender
{
    self.ExcludeSwitch.on = !self.ExcludeSwitch.isOn;
    [self updateGroupIncludeOrExcludeButtonTitle:self.includeSwitch.isOn];
    [self setFiltersWithGroups:allSelectedGroups emailsList:allSelectedEmails isExcluding:!self.includeSwitch.isOn];
    [self.SelectGroupsTableView reloadData];
}

- (IBAction)excludeSwitchStateChanged:(UISwitch *)sender
{
    self.includeSwitch.on = !self.includeSwitch.isOn;
    [self updateGroupIncludeOrExcludeButtonTitle:self.includeSwitch.isOn];
    [self setFiltersWithGroups:allSelectedGroups emailsList:allSelectedEmails isExcluding:!self.includeSwitch.isOn];
    [self.SelectGroupsTableView reloadData];
}

-(void)updateGroupIncludeOrExcludeButtonTitle:(BOOL)shouldInclude
{
    [self.groupsToIncludeOrExcludeButton setTitle:(shouldInclude ? @"Groups To Include" : @"Groups To Exclude") forState:UIControlStateNormal];
}

- (IBAction)OneToOneOnlySwitchStateChanged:(UISwitch *)sender
{
    [self resetFilters];
    if (sender.isOn)
    {
        if (self.groupsOnlySwitch.isOn)
        {
            self.groupsOnlySwitch.on = NO;
        }
        
        [allSelectedGroups removeAllObjects];
        
        NSPredicate * multiMemberPredicate = [NSPredicate predicateWithFormat:@"self.groupType == %d",TCGroupTypeOneToOne];
        NSError * fetcherror;
        allSelectedGroups = [[[Teamchat teamchatGroups:&fetcherror] filteredArrayUsingPredicate:multiMemberPredicate] mutableCopy];
        [self setFiltersWithGroups:allSelectedGroups emailsList:allSelectedEmails isExcluding:!self.includeSwitch.isOn];
    }
    else
    {
        [allSelectedGroups removeAllObjects];
    }
    [self.SelectGroupsTableView reloadData];
    [self enableOrDisableGroupsToIncludeButton];
}

- (IBAction)GroupsOnlySwitchChanged:(UISwitch *)sender
{
    [self resetFilters];
    if (sender.isOn)
    {
        if (self.oneToOneOnlySwitch.isOn)
        {
            self.oneToOneOnlySwitch.on = NO;
        }
        [allSelectedGroups removeAllObjects];
        NSPredicate * multiMemberPredicate = [NSPredicate predicateWithFormat:@"self.groupType == %d",TCGroupTypeMultiMemberGroup];
        NSError * fetcherror;
        allSelectedGroups = [[[Teamchat teamchatGroups:&fetcherror] filteredArrayUsingPredicate:multiMemberPredicate] mutableCopy];
        [self setFiltersWithGroups:allSelectedGroups emailsList:allSelectedEmails isExcluding:!self.includeSwitch.isOn];
    }
    else
    {
        [allSelectedGroups removeAllObjects];
    }
    [self.SelectGroupsTableView reloadData];
    [self enableOrDisableGroupsToIncludeButton];
}



#pragma handleGroupsToIncludeOrExcludeTapped

- (IBAction)handleGroupsToIncludeOrExcludeTapped:(UIButton *)sender
{
    [self resetFilters];
    ListTableViewController * listViewController = (ListTableViewController *)[[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"RoomsList"];
    listViewController.listType = ListTypeToFilterGroups;
    listViewController.delegate = self;
    [self.navigationController pushViewController:listViewController animated:YES];
}

- (IBAction)addEmail:(UIButton *)sender
{
    if (!self.emailTextField.text.length)
    {
        return;
    }
    __block BOOL contains = NO;
    [allSelectedEmails enumerateObjectsUsingBlock:^(NSString *  _Nonnull text, NSUInteger idx, BOOL * _Nonnull stop) {
        if ([text isEqualToString:self.emailTextField.text])
        {
            contains = *stop = YES;
        }
    }];
    if (contains)
    {
        [[[UIAlertView alloc] initWithTitle:@"Alert" message:@"email is already entered" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
        [self.emailTextField resignFirstResponder];
        return;
    }
    if (self.emailTextField.text)
    {
        [allSelectedEmails addObject:self.emailTextField.text];
        [self.SelectGroupsTableView reloadData];
        [self setFiltersWithGroups:allSelectedGroups emailsList:allSelectedEmails isExcluding:!self.includeSwitch];
        self.emailTextField.text = nil;
    }
    [self.emailTextField resignFirstResponder];
}

- (void)enableOrDisableGroupsToIncludeButton
{
    if (self.oneToOneOnlySwitch.isOn || self.groupsOnlySwitch.isOn)
    {
        self.groupsToIncludeOrExcludeButton.enabled = NO;
    }
    else
    {
        self.groupsToIncludeOrExcludeButton.enabled = YES;
    }
}

- (IBAction)resetButtonTapped:(UIButton *)sender
{
    [self resetFilters];
    self.includeSwitch.on = YES;
    self.ExcludeSwitch.on = NO;
    self.oneToOneOnlySwitch.on = NO;
    self.groupsOnlySwitch.on = NO;
    [allSelectedGroups removeAllObjects];
    [allSelectedEmails removeAllObjects];
    [self.SelectGroupsTableView reloadData];
}
- (IBAction)showChatRoomsTapped:(UIButton *)sender
{
    if (!self.groupsViewController)
    {
        self.groupsViewController = [Teamchat teamchatGroupsViewController];
    }
    if (self.groupsViewController)
    {
        [self presentViewController:self.groupsViewController animated:YES completion:nil];
    }
}
- (IBAction)dismissKeyBoardIfVissible:(UITapGestureRecognizer *)sender {
    
    if (self.emailTextField.isFirstResponder)
    {
        [self.emailTextField resignFirstResponder];
    }
}

#pragma mark - ListTableViewControllerDelegate

- (void)selectedGroups:(NSArray *)selectedGroups
{
    allSelectedGroups = [selectedGroups mutableCopy];
    [self.SelectGroupsTableView reloadData];
    [self setFiltersWithGroups:selectedGroups emailsList:allSelectedEmails isExcluding:!self.includeSwitch.isOn];
}

- (void)selectedContacts:(NSArray *)selectedContacts
{
    for (TCTeamchatContact * contact in selectedContacts)
    {
        [allSelectedEmails addObject:contact.email];
    }
    [self.SelectGroupsTableView reloadData];
    [self setFiltersWithGroups:allSelectedGroups emailsList:allSelectedEmails isExcluding:!self.includeSwitch.isOn];
}

-(void)resetFilters
{
    [Teamchat listOfTeamchatGroupsIncludingGroups:nil andAnyOneToOneUsersWithEmail:nil];
}

-(void)setFiltersWithGroups:(NSArray *)selectedGroups emailsList:(NSArray *)emails isExcluding:(BOOL)isExcluding
{
    if (isExcluding)
    {
        [Teamchat listOfTeamchatGroupsExcludingGroups:selectedGroups andAnyOneToOneUsersWithEmail:emails];
    }
    else
    {
        [Teamchat listOfTeamchatGroupsIncludingGroups:selectedGroups andAnyOneToOneUsersWithEmail:emails];
    }
}

@end
