//
//  RoomsListTableViewController.m
//  SDKTestApp
//
//  Created by Vidyalaxmi V Shenoy on 27/08/15.
//  Copyright (c) 2015 Robosoft. All rights reserved.
//

#import "ListTableViewController.h"
#import <TeamchatSDK/TeamchatSDK.h>
#import "SVProgressHUD.h"
#import "CustomContactsViewController.h"

@interface ListTableViewController () <UIActionSheetDelegate, ListControllerDelegate, CustomContactsViewControllerDelegate>

@property (nonatomic) NSArray *listArray;
@property (nonatomic) NSMutableArray *selectedContacts;
@property (nonatomic) NSIndexPath *selectedIndexPath;

@end

@implementation ListTableViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.tableView.tableFooterView = [[UIView alloc] initWithFrame:CGRectZero];
    
    if (self.listType == ListTypeServerContacts || self.listType == ListTypeToFilterGroups)
    {
        [self addBarButtonItems];
    }

    __weak ListTableViewController *weakSelf = self;
    
    [SVProgressHUD showWithMaskType:SVProgressHUDMaskTypeBlack];
    [Teamchat initWithCompletionHandler:^(BOOL success, NSError *error, NSString *errMsg) {
        [SVProgressHUD dismiss];
        if (success)
        {
            if (weakSelf.listType == ListTypeServerContacts)
            {
                [SVProgressHUD showWithMaskType:SVProgressHUDMaskTypeBlack];
                [Teamchat getTeamchatServerContactsWithCompletion:^(BOOL success, NSError *error, NSArray *fetchedContacts) {
                    weakSelf.listArray = fetchedContacts;
                    [weakSelf.tableView reloadData];
                    [SVProgressHUD dismiss];
                }];
            }
            else
            {
                NSError *groupsError;
                NSArray *groups = [Teamchat teamchatGroups:&groupsError];
                if (groups)
                {
                    if (weakSelf.listType == ListTypeToAddContactsToGroup)
                    {
                        // Get groups where current user is admin
                        NSIndexSet *indexSet = [groups indexesOfObjectsPassingTest:^BOOL(TeamchatGroup *group, NSUInteger idx, BOOL * _Nonnull stop) {
                            NSError *adminError;
                            TeamchatGroupMember *adminMember = [TeamchatGroupController getAdminOfGroup:group error:&adminError];
                            return ([adminMember.memberID isEqualToString:[TeamchatUser userID]] && ![group isGroupOneToOne]);
                        }];
                        
                        if (indexSet.count)
                        {
                            weakSelf.listArray = [groups objectsAtIndexes:indexSet];
                        }
                    }
                    else
                    {
                        weakSelf.listArray = groups;
                    }
                    
                    [weakSelf.tableView reloadData];
                }
                else
                {
                    [weakSelf showFailureAlertWithMessage:[groupsError localizedDescription]];
                }
                
            }
        }
        else
        {
            [weakSelf showFailureAlertWithMessage:[error localizedDescription]];
        }
    }];
}

#pragma mark - Action methods

- (void)addBarButtonItems
{
    UIBarButtonItem *cancelButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel
                                                                                      target:self action:@selector(cancelButtonTapped:)];
    self.navigationItem.leftBarButtonItem = cancelButtonItem;
    
    UIBarButtonItem *doneButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone
                                                                                    target:self action:@selector(doneButtonTapped:)];
    self.navigationItem.rightBarButtonItem = doneButtonItem;
}

- (void)cancelButtonTapped:(id)sender
{
    if (self.presentingViewController)
    {
        [self dismissViewControllerAnimated:YES completion:NULL];
    }
    else
    {
        [self.navigationController popViewControllerAnimated:YES];
    }
}

- (void)doneButtonTapped:(id)sender
{
    if ([self.delegate respondsToSelector:@selector(selectedContacts:)] && !(self.listType == ListTypeToFilterGroups))
    {
        [self.delegate selectedContacts:self.selectedContacts];
    }
    
    if ([self.delegate respondsToSelector:@selector(selectedGroups:)] && self.listType == ListTypeToFilterGroups)
    {
        [self.delegate selectedGroups:self.selectedContacts];
    }
    
    if (self.presentingViewController)
    {
        [self dismissViewControllerAnimated:YES completion:nil];
    }
    else
    {
        [self.navigationController popViewControllerAnimated:YES];
    }
}

- (void)showFailureAlertWithMessage:(NSString *)alertMessage
{
    [[[UIAlertView alloc] initWithTitle:@"Failed" message:alertMessage
                               delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil]
     show];
}

#pragma mark - Table view data source

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.listArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"GroupsListCell" forIndexPath:indexPath];

    switch (self.listType)
    {
        case ListTypeServerContacts:
        {
            TCTeamchatContact *serverContact = self.listArray[indexPath.row];
            cell.textLabel.text = serverContact.profileName;
            cell.detailTextLabel.text = serverContact.email;
            break;
        }
            
        case ListTypeToAddContactsToGroup:
        case ListTypeToShowChatRoom:
        case ListTypeToFilterGroups:
        {
            TeamchatGroup *group = self.listArray[indexPath.row];
            
            cell.textLabel.text = [NSString stringWithFormat:@"Group Name : %@", group.name];
            cell.detailTextLabel.text = [NSString stringWithFormat:@"Group ID : %@", group.groupID];
            if (self.listType == ListTypeToAddContactsToGroup)
            {
                [self addCustomAccessoryViewToCell:cell];
                [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
            }
            if (self.listType == ListTypeToFilterGroups)
            {
                BOOL isSelected = [self.selectedContacts containsObject:group];
                cell.accessoryType = isSelected ? UITableViewCellAccessoryCheckmark : UITableViewCellAccessoryNone;
                [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
            }
            break;
        }
            
        default:
            break;
    }
        return cell;
}

- (void)addCustomAccessoryViewToCell:(UITableViewCell *)cell
{
    UIImage *image = [UIImage imageNamed:@"add_icon"];
    
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    CGRect frame = CGRectMake(0.0, 0.0, 40, 40);
    button.frame = frame;
    [button setImage:image forState:UIControlStateNormal];
    
    [button addTarget:self action:@selector(accessoryButtonTapped:withEvent:)  forControlEvents:UIControlEventTouchUpInside];
    button.backgroundColor = [UIColor clearColor];
    cell.accessoryView = button;
}

- (void) accessoryButtonTapped:(UIControl *)button withEvent:(UIEvent *)event
{
    NSIndexPath * indexPath = [self.tableView indexPathForRowAtPoint:[[[event touchesForView: button] anyObject] locationInView: self.tableView]];
    self.selectedIndexPath = indexPath;
    
    if ( indexPath == nil )
        return;
    
    [self tableView:self.tableView accessoryButtonTappedForRowWithIndexPath:indexPath];
}

#pragma mark - UITableViewDelegate methods

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    switch (self.listType)
    {
        case ListTypeToShowChatRoom:
        {
            [Teamchat setLoggingEnabled:YES];
            [tableView deselectRowAtIndexPath:indexPath animated:YES];
            TeamchatGroup *group = self.listArray[indexPath.row];
            
            [Teamchat initWithCompletionHandler:^(BOOL success, NSError *error, NSString *errMsg) {
                if (success)
                {
                    UIViewController *groupViewController = [Teamchat createTeamchatWindowViewControllerWithGroup:group];
                    if (groupViewController)
                    {
                        [self.navigationController pushViewController:groupViewController animated:YES];
                    }
                }
                else
                {
                    [self showFailureAlertWithMessage:[error localizedDescription]];
                }
            }];

            break;
        }

        case ListTypeServerContacts:
        {
            TCTeamchatContact *selectedContact = self.listArray[indexPath.row];
            BOOL isSelected = [self.selectedContacts containsObject:selectedContact];
            
            UITableViewCell *selectedCell = [tableView cellForRowAtIndexPath:[tableView indexPathForSelectedRow]];
            selectedCell.accessoryType = isSelected ? UITableViewCellAccessoryNone : UITableViewCellAccessoryCheckmark;
            
            if (isSelected)
            {
                [self.selectedContacts removeObject:selectedContact];
            }
            else
            {
                if (!self.selectedContacts)
                {
                    self.selectedContacts = [NSMutableArray array];
                }
                [self.selectedContacts addObject:selectedContact];
            }
            
            break;
        }
            
        case ListTypeToFilterGroups:
        {
            TeamchatGroup *selectedGroup = self.listArray[indexPath.row];
            BOOL isSelected = [self.selectedContacts containsObject:selectedGroup];
            
            UITableViewCell *selectedCell = [tableView cellForRowAtIndexPath:[tableView indexPathForSelectedRow]];
            selectedCell.accessoryType = isSelected ? UITableViewCellAccessoryNone : UITableViewCellAccessoryCheckmark;
            
            if (isSelected)
            {
                [self.selectedContacts removeObject:selectedGroup];
            }
            else
            {
                if (!self.selectedContacts)
                {
                    self.selectedContacts = [NSMutableArray array];
                }
                [self.selectedContacts addObject:selectedGroup];
            }
            
            break;
        }
            
        default:
            break;
    }
    
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (void)tableView:(UITableView *)tableView accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath
{
    [self showContactAdditionOptions];
}

- (void)showContactAdditionOptions
{
    UIActionSheet *actionSheet = [[UIActionSheet alloc] initWithTitle:nil delegate:self cancelButtonTitle:@"Cancel" destructiveButtonTitle:nil otherButtonTitles:@"Add custom contacts",@"Add Teamchat contacts", nil];
    [actionSheet showInView:self.view];
}

#pragma mark - UIActionSheetDelegate methods

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex
{
    if (buttonIndex != actionSheet.cancelButtonIndex)
    {
        switch (buttonIndex)
        {
            case 0:
                [self showCustomContactsView];
                break;
                
            case 1:
                [self showServerContacts];
                break;
                
            default:
                break;
        }
    }
}

- (void)showCustomContactsView
{
    CustomContactsViewController *customContactsViewController = [self.storyboard instantiateViewControllerWithIdentifier:@"CustomContactsViewController"];
    customContactsViewController.customContactViewType = CustomContactViewTypeForAddingContactsToGroup;
    [customContactsViewController configureWithCustomContacts:nil delegate:self];
    
    UINavigationController *navigationController = [[UINavigationController alloc] initWithRootViewController:customContactsViewController];
    [self presentViewController:navigationController animated:YES completion:nil];
}

- (void)showServerContacts
{
    ListTableViewController *serverContactsViewController = [self.storyboard instantiateViewControllerWithIdentifier:@"RoomsList"];
    serverContactsViewController.listType = ListTypeServerContacts;
    serverContactsViewController.delegate = self;
    
    UINavigationController *navController = [[UINavigationController alloc] initWithRootViewController:serverContactsViewController];
    [self presentViewController:navController animated:YES completion:nil];
}

- (void)addSelectedContactsToGroup:(NSArray *)selectedContacts
{
    if (self.selectedIndexPath && self.selectedIndexPath.row < self.listArray.count)
    {
        TeamchatGroup *group = self.listArray[self.selectedIndexPath.row];
        
        [SVProgressHUD showWithMaskType:SVProgressHUDMaskTypeBlack];
        
        [TeamchatGroupController addMembers:selectedContacts toGroup:group completion:^(BOOL success, NSError *error, NSString *msg) {
            
            [SVProgressHUD dismiss];
            
            NSString *alertTitle = success ? @"Success" : @"Failed";
            NSString *alertMessage = success ? @"Member addition successful" : error ? [error localizedDescription] : msg ? : nil;
            [[[UIAlertView alloc] initWithTitle:alertTitle message:alertMessage
                                       delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
        }];
    }
}

#pragma mark - ListControllerDelegate methods

- (void)selectedContacts:(NSArray *)selectedContacts
{
    if (selectedContacts.count)
    {
        [self addSelectedContactsToGroup:selectedContacts];
    }
}

#pragma mark - CustomContactsViewControllerDelegate methods

- (void)customContacts:(NSArray *)customContacts
{
    if (customContacts.count)
    {
        [self addSelectedContactsToGroup:customContacts];
    }
}

@end
