//
//  LoginViewController.m
//  SDKTestApp
//
//  Created by Vidyalaxmi V Shenoy on 25/08/15.
//  Copyright (c) 2015 Robosoft. All rights reserved.
//

#import "LoginViewController.h"

@interface LoginViewController ()

@property (weak, nonatomic) IBOutlet UITextField *userEmailTextField;
@property (weak, nonatomic) IBOutlet UILabel *userIDLabel;
@property (weak, nonatomic) IBOutlet UITextField *userIDTextField;
@property (weak, nonatomic) IBOutlet UILabel *authTokenLabel;
@property (weak, nonatomic) IBOutlet UITextField *authTokenTextField;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *loginButtonVerticalSpaceConstraint;
@property (weak, nonatomic) IBOutlet UIButton *loginButton;
@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;
@property (nonatomic) UITextField *activeTextField;

@end

@implementation LoginViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self hideViewsBasedOnLoginType];
    
    [self addInputAccessoryViewForTextField:self.userIDTextField];
    
    if (self.logionOptionType == LoginOptionTypeDefault)
    {
        [self.loginButton setTitle:@"Login" forState:UIControlStateNormal];
    }
    
    [self registerToKeyboardNotifications];
}

- (void)hideViewsBasedOnLoginType
{
    switch (self.logionOptionType)
    {
        case LoginOptionTypeTeamchatLoginWithPrefilledEmail:
        {
            self.userIDLabel.hidden = self.userIDTextField.hidden = YES;
            self.authTokenLabel.hidden = self.authTokenTextField.hidden = YES;
            self.loginButtonVerticalSpaceConstraint.constant = 30;
            break;
        }
            
        default:
            break;
    }
}

- (void)addInputAccessoryViewForTextField:(UITextField *)textField
{
    CGRect accessFrame = CGRectMake(0.0, 0.0, 768.0, 44.0);
    UIView *inputAccessoryView = [[UIView alloc] initWithFrame:accessFrame];
    
    UIButton *doneButton = [[UIButton alloc] initWithFrame:CGRectMake([UIScreen mainScreen].bounds.size.width - 75, 0.0, 90, 44.0)];
    [doneButton setTitle:@"Set"
                forState:UIControlStateNormal];
    
    [doneButton addTarget:self
                   action:@selector(setButtonTapped:)
         forControlEvents:UIControlEventTouchUpInside];
    
    [inputAccessoryView addSubview:doneButton];
    inputAccessoryView.backgroundColor = [UIColor colorWithRed:156.f/255.f green:163.f/255.f blue:177.f/255.f alpha:1];
    textField.inputAccessoryView = inputAccessoryView;
}

- (void)registerToKeyboardNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardDidShow:)
                                                 name:UIKeyboardDidShowNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillBeHidden:)
                                                 name:UIKeyboardWillHideNotification
                                               object:nil];
}

#pragma mark - Action methods

- (void)keyboardDidShow:(NSNotification *)notification
{
    NSDictionary* info = [notification userInfo];
    CGRect kbRect = [[info objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue];
    kbRect = [self.view convertRect:kbRect fromView:nil];
    
    UIEdgeInsets contentInsets = UIEdgeInsetsMake(0.0, 0.0, kbRect.size.height, 0.0);
    self.scrollView.contentInset = contentInsets;
    self.scrollView.scrollIndicatorInsets = contentInsets;
    
    CGRect aRect = self.view.frame;
    aRect.size.height -= kbRect.size.height;
    if (!CGRectContainsPoint(aRect, self.activeTextField.frame.origin) ) {
        [self.scrollView scrollRectToVisible:self.activeTextField.frame animated:YES];
    }
}

- (void)keyboardWillBeHidden:(NSNotification *)notification
{
    UIEdgeInsets contentInsets = UIEdgeInsetsZero;
    self.scrollView.contentInset = contentInsets;
    self.scrollView.scrollIndicatorInsets = contentInsets;
}

- (void)setButtonTapped:(id)sender
{
    [self.view endEditing:YES];
}

- (IBAction)viewTapHandler:(id)sender
{
    [self.view endEditing:YES];
}

- (IBAction)cancelButtonTapped:(id)sender
{
    [self.view endEditing:YES];
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)showLoginScreenTapped:(id)sender
{
    switch (self.logionOptionType)
    {
        case LoginOptionTypeDefault:
        {
            if (self.userIDTextField.text.length && self.userEmailTextField.text.length && self.authTokenTextField.text.length)
            {
                [self dismissViewControllerAnimated:NO completion:^{
                    if ([self.delegate respondsToSelector:@selector(showLoginScreenForLoginOption:email:userID:authToken:)])
                    {
                        [self.delegate showLoginScreenForLoginOption:self.logionOptionType
                                                               email:self.userEmailTextField.text
                                                              userID:self.userIDTextField.text
                                                           authToken:self.authTokenTextField.text];
                    }
                }];
            }
            else
            {
                [self showFailureAlert];
            }
            
            break;
        }
         case LoginOptionTypeTeamchatLoginWithPrefilledEmail:
        {
            if (self.userEmailTextField.text.length)
            {
                [self dismissViewControllerAnimated:NO completion:^{
                    if ([self.delegate respondsToSelector:@selector(showLoginScreenForLoginOption:email:userID:authToken:)])
                    {
                        [self.delegate showLoginScreenForLoginOption:self.logionOptionType
                                                               email:self.userEmailTextField.text
                                                              userID:nil
                                                           authToken:nil];
                    }
                }];
            }
            else
            {
                [self showFailureAlert];
            }

            break;
        }
        default:
            break;
    }
}

- (void)showFailureAlert
{
    [[[UIAlertView alloc] initWithTitle:@"Failed" message:@"Please fill all the fields" delegate:nil
                                cancelButtonTitle:@"OK"
                                otherButtonTitles:nil, nil] show];
}

#pragma mark - UITextFieldDelegate methods

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    return YES;
}

- (void)textFieldDidBeginEditing:(UITextField *)textField
{
    self.activeTextField = textField;
}

- (void)textFieldDidEndEditing:(UITextField *)textField
{
    self.activeTextField = nil;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

@end
